/*
 * This file is part of Deliantra, the Roguelike Realtime MMORPG.
 * 
 * Copyright (©) 2005,2006,2007,2008 Marc Alexander Lehmann / Robin Redeker / the Deliantra team
 * Copyright (©) 2002,2007 Mark Wedel & Crossfire Development Team
 * Copyright (©) 1992,2007 Frank Tore Johansen
 * 
 * Deliantra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * The authors can be reached via e-mail to <support@deliantra.net>
 */

/*
 * Command parser
 */

#include <cctype>

#include <global.h>
#include <commands.h>
#include <sproto.h>

/* Added times to all the commands.  However, this was quickly done,
 * and probably needs more refinements.  All socket and DM commands
 * take 0 time.
 */

/*
 * Normal game commands
 */
CommArray_s Commands[] = {
  {"party", command_party, 1.0},
  {"gsay",  command_gsay, 1.0},

  {"apply",        command_apply, 1.0},        /* should be variable */
  {"cast",         command_cast, 1.0},  /* Is this right? */
  {"dm",           command_dm, 0.0},
  {"dmhide",       command_dmhide, 0.0},      /* Like dm, but don't tell a dm arrived, hide player */
  {"drop",         command_drop, 1.0},
  {"dropall",      command_dropall, 0.5 / TICK},
  {"examine",      command_examine, 0.5 / TICK},
  {"fix_me",       command_fix_me, 0.0},
  {"get",          command_take, 1.0},
  {"invoke",       command_invoke, 1.0},
  {"killpets",     command_kill_pets, 1.0},
  {"motd",         command_motd, 0.5 / TICK},
  {"pickup",       command_pickup, 1.0},
  {"prepare",      command_prepare, 1.0},
  {"resistances",  command_resistances, 1.0},
  {"rotateshoottype", command_rotateshoottype, 1.0},
  {"skills",       command_skills, 0.5 / TICK},      /* shows player list of skills */
  {"use_skill",    command_uskill, 1.0},
  {"ready_skill",  command_rskill, 1.0},
  {"search-items", command_search_items, 1.0},
  {"showpets",     command_showpets, 1.0},
  {"statistics",   command_statistics, 1.0},
  {"take",         command_take, 1.0},
  {"throw",        command_throw, 1.0},
  {"time",         command_time, 0.24 / TICK},
  {"title",        command_title, 0.5 / TICK},
  {"bowmode",      command_bowmode, 1.0},
  {"version",      command_version, 1.0},

  // movement commands all do their own speed management
  {"stay",      command_stay,      0.0},
  {"north",     command_north,     0.0},
  {"east",      command_east,      0.0},
  {"south",     command_south,     0.0},
  {"west",      command_west,      0.0},
  {"northeast", command_northeast, 0.0},
  {"southeast", command_southeast, 0.0},
  {"southwest", command_southwest, 0.0},
  {"northwest", command_northwest, 0.0},
};
const int CommandsSize = sizeof (Commands) / sizeof (CommArray_s);

/*
 * Wizard commands (for both)
 */
CommArray_s WizCommands[] = {
  {"abil",     command_abil, 0.0},
  {"addexp",   command_addexp, 0.0},
  {"arrest",   command_arrest, 0.0},
  {"create",   command_create, 0.0},
  {"debug",    command_debug, 0.0},
  {"dump",     command_dump, 0.0},
  {"dumpbelow", command_dumpbelow, 0.0},
  {"forget_spell", command_forget_spell, 0.0},
  {"free",     command_free, 0.0},
  {"freeze",   command_freeze, 0.0},
  {"hide",     command_hide, 0.0},
  {"insert_into", command_insert_into, 0.0},
  {"inventory",   command_inventory, 0.0},
  {"invisible", command_invisible, 0.0},
  {"learn_special_prayer", command_learn_special_prayer, 0.0},
  {"learn_spell", command_learn_spell, 0.0},
  {"nodm",     command_nowiz, 0.0},
  {"nowiz",    command_nowiz, 0.0},
  {"patch",    command_patch, 0.0},
  {"remove",   command_remove, 0.0},
  {"set_god",  command_setgod, 0.0},
  {"stack_clear", command_stack_clear, 0.0},
  {"stack_list",  command_stack_list, 0.0},
  {"stack_pop",   command_stack_pop, 0.0},
  {"stack_push",  command_stack_push, 0.0},
  {"stats",    command_stats, 0.0},
  {"summon",   command_summon, 0.0},
  //{"overlay_save", command_save_overlay, 0.0},
};
const int WizCommandsSize = sizeof (WizCommands) / sizeof (CommArray_s);

static int
compare_A (const void *a, const void *b)
{
  return strcmp (((CommArray_s *)a)->name, ((CommArray_s *)b)->name);
}

void
init_commands (void)
{
  qsort (Commands, CommandsSize, sizeof (CommArray_s), compare_A);
  qsort (WizCommands, WizCommandsSize, sizeof (CommArray_s), compare_A);
}

